@@ -4,7 +4,10 @@
 
 package Circle;
 
-our $VERSION = '0.01';
+use strict;
+use warnings;
+
+our $VERSION = '0.02';
 
 1;
 
